/* -*- Mode:C++; c-file-style:"gnu"; indent-tabs-mode:nil; -*- */
/*
 * Copyright (c) 2015 Fraunhofer FKIE
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation;
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Author: Christoph Fuchs <christoph.fuchs@fkie.fraunhofer.de>
 */

#ifndef RADIOWATCHER_HELPER_H
#define RADIOWATCHER_HELPER_H

#include "ns3/node-container.h"
#include "ns3/nstime.h"
#include "ns3/double.h"
#include "ns3/ptr.h"
#include "ns3/application-container.h"
#include "ns3/ipv4-address.h"
#include "ns3/mac48-address.h"
#include "ns3/tap-bridge-module.h"
#include "ns3/radiowatcher.h"

namespace ns3 {

/**
 * \brief create RadioWatcher instances and connect them to wifi devices
 *
 *
 */
class RadioWatcherHelper
{
public:
  RadioWatcherHelper ();
  ApplicationContainer Install(NodeContainer c);
  
  /**
  * \param address the IP address for the L2StatServers
  *
  * Each RadioWatcher periodically sends statistics over its
  * L2Statserver which is bound to the specified local IP address.
  */
  void SetLocalSocketAddress (char const *address);

  /**
  * \param address the (optional) IPv6 address for the L2StatServers
  *
  * If an IPv6 address is set, each RadioWatcher periodically sends 
  * its statistics also using the IPv6 address. Port numbers are the
  * same as for IPv4.
  */
  void SetLocalSocketv6Address (char const *address);

  /**
  * \param port starting port number for the L2StatServers
  *
  * A RadioWatcher binds its L2Statserver to a local port wich is
  * determined by adding the nodeid of the RadioWatcher's node to the
  * base port number specified here.
  */
  void SetLocalSocketBasePort (uint16_t port);
  
  void SetInterval (double interval);

private:
  Ptr<RadioWatcher> EnableRadioWatcher (uint32_t nodeid, uint32_t deviceid, Ptr<NetDevice> nd, Mac48Address local_address);
  Ptr<RadioWatcher> EnableRadioWatcher (uint32_t nodeid, uint32_t deviceid, Ptr<NetDevice> nd);
  void StartRadioWatcher (Ptr<RadioWatcher> radiowatcher, uint32_t nodeid);
  Time m_interval;
  Ipv4Address m_localSocketAddress;
  Ipv6Address m_localSocketv6Address;
  bool m_useIPv6;
  uint16_t m_localSocketBasePort;
}; // class RadioWatcherHelper


} // namespace ns3

#endif /* RADIOWATCHER_HELPER_H */
